//<c>   Copyright 1995-2000 by Gerry J. Danen; all rights reserved.

//<t>   Convert an FTN-Compliant NodeList to Fixed Format

/*      Usage:  FNI file_in file_out

        Full pathnames are supported for the filenames.

        Purpose:  To read a single nodelist and translate it from the raw St.
        Louis nodelist to a fixed file format:

        zone:net/node    bbs name    sysop name    location    phone number

        If you want to change this program, go ahead.  All I ask is that you
        e-mail the changed version to gdanen@bigfoot.com.

        Copyright 1995-2000 by Gerry Danen; all rights reserved.
        Permission to copy granted, provided all files are included in the
        archive: FILE_ID.DIZ, FNI.C, FNI.EXE, NODELIST.000, TEST.BAT.

        Previous versions have been distributed that required the DIS libraries.
        Those libraries applied only to MS compilers. Hopefully you can compile
        this program under any compiler, on any platform. You may have to change
        some header file names and/or function names...
*/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <types.h>
#include <ctype.h>


//      definitions from gd_tools.h -- included for ease of porting
typedef signed int      BOOL ;                  // fast integer for TRUE/FALSE and OK/ERROR
typedef signed int      INT16 ;                 // 16-bit integer / signed
typedef unsigned char   UCHAR ;                 // 8-bit char / unsigned
typedef unsigned char * STRING ;                // string pointer
#define LOOP(var,from,to,incr)  for ( (var)=(from) ; (var) <= (to) ; (var)=(var)+(incr) )
#define TRUE            (1)
#define FALSE           (0)
#define EOS             '\0'                    // end of string
#define FS_LEN          260                     // in NetWare longer than DOS
#define GD_ERROR_NONE   0                       // no error, everything OK
#define GD_ERROR_HELP   1                       // no error, but exit via help message
#define GD_ERROR_CANNOT_OPEN_FOR_WRITE  203     // cannot open file for write operations


#define DeBug 0


#define pgVERS_ "V2.31"
#define pgTITL_ "FTN-Compliant NodeList Interpreter"
#define pgCOPR_ "1995"


#define BUFSIZE 10000
UCHAR   iFile[FS_LEN], oFile[FS_LEN], work[BUFSIZE+2], *w, word[81],
        sZone[81], sHost[81], sNode[81], sZoneHostNode[81] ;
long    lDown=0, lHold=0, lHost=0, lHub=0, lPvt=0, lRegion=0, lZone=0, lRecs=0 ;
int     iZHN=0, iBBS=0, iSysOp=0, iLocn=0, iPhone=0 ;
FILE    *iFh, *oFh ;


void    show_help_and_quit( void )
{
    fprintf( stderr,
             "\nThis program converts an FTN-Compliant Nodelist to a text format\n\n"
             "\tUsage:  FNI file_in file_out\n\n" );
    exit( GD_ERROR_HELP );
}


STRING  s_remove_spaces( UCHAR s[] )
{
    INT16       fx, tx, slen ;

    slen = strlen(s);
    for ( tx=0, fx=0 ; tx < slen ; )
    {
        if ( fx < slen )
        {
            if ( isspace(s[fx]) )
                fx++ ;                  // white space - skip
            else
                s[tx++] = s[fx++] ;     // nonwhite - copy & move both indexes
        }
        else
        {
            s[tx++] = EOS ;             // no more "from" chars, so set rest to EOS
        }
    }
    return( s );
}


INT16   s_length( UCHAR str[] )
{
    register int i ;
    int l=0, x=strlen(str)-1;

    LOOP( i, 0, x, 1 )
    {
        if ( (str[i] != EOS) && (! isspace(str[i])) )
            l = i+1 ;
    }
    return( (INT16) l );
}


BOOL    s_equal( STRING s1, STRING s2 )
{
    if ( s_length(s1) != s_length(s2) )         // not equal length
        return( FALSE );
    if ( _strnicmp( s1, s2, s_length(s1) ) == 0 )
        return( TRUE );
    else
        return( FALSE );
}


STRING  s_copy( UCHAR dest[], UCHAR src[], INT16 maxchar )
/*
        Copy a string from "src" to "dest".  At the most, "maxchar-1"
        characters will be copied, in order to accomodate the terminating
        EOS. "dest" must have been declared/allocated "maxchar" characters
        long.
*/
{
    register    int     n ;
    register    UCHAR   *d ;
    INT16       l ;

    d = dest ;
    l = maxchar - 1 ;
    n = strlen( src );
    if ( n > l )
        n = l ;
    memmove( d, src, n );
    dest[n] = EOS ;
    return( d );                                // destination string
}


void    s_ltrim( STRING string )
{
    register    UCHAR *src ;

    if ( strlen(string) )
    {
        src = string ;
        while ( isspace(*src) )
            src++ ;
        s_copy( string, src, strlen(src)+1 );
    }
}

void    s_rtrim( STRING string )
{
    register    UCHAR *last ;

    last = string + strlen(string) ;            // get to end of string
    while ( ((*last == EOS) || (isspace(*last))) && (last >= string) )
    {
        *last-- = EOS ;
    }
}

void    s_trim( STRING string )
{
    s_rtrim( string );
    s_ltrim( string );
}


STRING  s_rem_word( STRING str )                // remove first word in string
{
    register int i ;

    s_ltrim( str );                             // get rid of leading spaces
    if ( str[0] != EOS )
    {
        i = 0 ;
        while ( (str[i]) && (! isspace(str[i])) )
        {
            str[i++] = ' ' ;
        }
    }
    s_ltrim( str );
    return ( str );
}


STRING  s_get_word( STRING dest, STRING str )   // get first word in string
{
    UCHAR       ch, *word = dest ;

    if ( strlen(str) )
    {
        while ( (ch = *str++) && isspace(ch) )
            ;                                   // skip spaces
        str-- ;
        while ( (ch = *str++) && (! isspace(ch)) )
        {
            *dest++ = ch ;                      // copy non-space chars
        }
    }
    *dest = EOS ;                               // terminate string
    return ( word );
}


void    fix_underscore( UCHAR s[] )
{
    static int x ;

    LOOP(x, 0, ((int)strlen(s)), 1)             // change all "_" to spaces
    {
        if ( s[x] == '_' )
            s[x] = ' ' ;
    }
}


void    process_unknown( void )
{
    strupr( word );
    work[38] = EOS ;
    fprintf( stderr, "\rUnknown entry \"%s\" at line %-6ld-> %s\n", word, lRecs, work );
}


void    process_node_entry( void )
{
    static UCHAR sBBS[81], sSysOp[81], sLocn[81], sPhone[81], sSpeed[81], sFlags[81] ;

    s_get_word( sBBS, work );                   // extract BBS name
    s_rem_word( work );                         // remove from buffer
    s_get_word( sLocn, work );                  // extract location
    s_rem_word( work );                         // remove from buffer
    s_get_word( sSysOp, work );                 // extract SysOp name
    s_rem_word( work );                         // remove from buffer
    s_get_word( sPhone, work );                 // extract phone number
    s_rem_word( work );                         // remove from buffer
    fix_underscore( sBBS   );
    fix_underscore( sLocn  );
    fix_underscore( sSysOp );
    if ( s_equal(sPhone, "-unpublished-") )
        strcpy( sPhone, "-" );
    s_get_word( sSpeed, work );                 // extract baud rate
    s_rem_word( work );                         // remove from buffer
    s_copy( sFlags, work, sizeof(work) );       // flags

#if DeBug
//  the field sizes collected here will determine the %-xx.xxs parts
//  in the fprintf below.
    if ( (int)strlen(sZoneHostNode) > iZHN )
        iZHN = (int)strlen(sZoneHostNode);
    if ( (int)strlen(sZoneHostNode) > 11 )
        printf( "\rZHN>12: %s %s\n", sZoneHostNode, sBBS );
    if ( (int)strlen(sBBS) > iBBS )
        iBBS = (int)strlen(sBBS);
    if ( (int)strlen(sSysOp) > iSysOp )
        iSysOp = (int)strlen(sSysOp);
    if ( (int)strlen(sLocn) > iLocn )
        iLocn = (int)strlen(sLocn);
    if ( (int)strlen(sPhone) > iPhone )
        iPhone = (int)strlen(sPhone);
#endif

    sprintf( sZoneHostNode, "%s:%s/%s", sZone, sHost, sNode );
    fprintf( oFh, "%-12.12s%-59.59s%-47.47s%-36.36s%-20.20s%-8.8s%s\n",
                  sZoneHostNode, sBBS, sSysOp, sLocn, sPhone, sSpeed, sFlags );
}


void    process_zone( void )
{
    s_get_word( sZone, work );                  // extract zone
    s_rem_word( work );                         // remove from buffer
    strcpy( sHost, sZone );                     // set host to same value
    strcpy( sNode, "0" );                       // set node to 0
    process_node_entry();
}


void    process_host( void )
{
    s_get_word( sHost, work );                  // extract host
    s_rem_word( work );                         // remove from buffer
    strcpy( sNode, "0" );                       // set node to 0
    process_node_entry();
}


void    process_node( void )
{
    s_get_word( sNode, work );                  // extract node number
    s_rem_word( work );                         // remove from buffer
    process_node_entry();
}


void    main( int argc, char *argv[] )
{
    int x ;

//  tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
//  logon(1);

    if ( argc != 3 )
        show_help_and_quit();

    s_copy( iFile, (UCHAR *) argv[1], FS_LEN );
    s_copy( oFile, (UCHAR *) argv[2], FS_LEN );
    strupr( (char *) iFile );
    strupr( (char *) oFile );

    if ( ( iFh = fopen( iFile, "r" ) ) == NULL )
    {
//      logon(1);
        fprintf( stderr, "\n\nFile %s not found\a\n", iFile );
        show_help_and_quit();
    }

//  oFh = fl_create_or_replace_tty( oFh, oFile, "w", YES, NO );

    oFh = fopen( oFile, "w" );
    if ( oFh == NULL )
    {
        fprintf( stderr, "\nCannot open %s for writing\n", oFile );
        exit( GD_ERROR_CANNOT_OPEN_FOR_WRITE );
    }

    fprintf( stderr, "Processing node list %s\n", iFile );

    while ( (w = fgets( work, BUFSIZE, iFh )) )
    {
        s_trim( work );                         // remove leading/trailing spaces
        if ( work[0] != ';' )                   // ignore comments
        {
            s_remove_spaces( work );            // remove embedded spaces
            lRecs ++ ;                          // count data records
            if ( ! (lRecs % 100L) )
                fprintf( stderr, "\r%ld", lRecs ); // show progress
            LOOP(x, 0, ((int)strlen(work)), 1)  // change all commas to spaces
            {
                if ( work[x] == ',' )
                    work[x] = ' ' ;
            }
            if ( work[0] == ' ' )               // first char _was_ a ","
            {
                process_node();
            }
            else                                // we might have a special entry
            {                                   //  .. Down, Hold, Host, Hub, Pvt, Region, Zone
                s_get_word( word, work );       // get first word in string
                s_rem_word( work );             // remove first word in string
                if ( s_equal(word, "down") )
                {
                    lDown ++ ;
                }
                else
                if ( s_equal(word, "hold") )
                {
                    lHold ++ ;
                    process_node();
                }
                else
                if ( s_equal(word, "host") )
                {
                    lHost ++ ;
                    process_host();
                }
                else
                if ( s_equal(word, "hub") )
                {
                    lHub ++ ;
                    process_node();
                }
                else
                if ( s_equal(word, "pvt") )
                {
                    lPvt ++ ;
                    process_node();
                }
                else
                if ( s_equal(word, "region") )
                {
                    lRegion ++ ;
                    process_host();
                }
                else
                if ( s_equal(word, "zone") )
                {
                    lZone ++ ;
                    process_zone();
                }
                else
                    process_unknown();
            }
        }
    }
    _fcloseall();
    fprintf( stderr, "\r%10ld data records processed:\n"
                     "\t\t%7ld nodes down,\n"
                     "\t\t%7ld nodes on hold,\n"
                     "\t\t%7ld host (net) nodes,\n"
                     "\t\t%7ld hub nodes,\n"
                     "\t\t%7ld private nodes,\n"
                     "\t\t%7ld regions,\n"
                     "\t\t%7ld zones.\n",
             lRecs, lDown, lHold, lHost, lHub, lPvt, lRegion, lZone );

#if DeBug
    fprintf( stderr,
             "\nDebug Information:\n"
             "\tlongest ZHN address %d\n"
             "\tlongest BBS name    %d\n"
             "\tlongest SysOp name  %d\n"
             "\tlongest Location    %d\n"
             "\tlongest Phone #     %d\n", iZHN, iBBS, iSysOp, iLocn, iPhone );
#endif

    fprintf( stderr,
             "\n\n\aPermission to copy FNI is granted, provided all files are included in\n"
             "the archive: FILE_ID.DIZ, FNI.C, FNI.EXE, NODELIST.000, TEST.BAT.\n\n" );

    fprintf( stderr,
             "If you want to change this program, go ahead. All I ask is that you\n"
             "e-mail the changed version to gdanen@bigfoot.com.\n\n" );

    exit( GD_ERROR_NONE );
}
